﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Syncfusion.Windows.Tools.Controls;
using Syncfusion.Windows.Shared;
using System.Diagnostics;
using Syncfusion.Windows.Controls.RichTextBoxAdv;
using System.ComponentModel;

namespace RichTextBoxAdvSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        #region Properties
        /// <summary>
        /// Gets or Sets the font family source.
        /// </summary>
        public List<string> FontFamilySource
        {
            get
            {
                return (List<string>)GetValue(FontFamilySourceProperty);
            }
            set
            {
                SetValue(FontFamilySourceProperty, value);
            }
        }
        /// <summary>
        /// Gets or Sets the font size source.
        /// </summary>
        public double[] FontSizeSource
        {
            get
            {
                return (double[])GetValue(FontSizeSourceProperty);
            }
            set
            {
                SetValue(FontSizeSourceProperty, value);
            }
        }
        /// <summary>
        /// Gets or Sets the list view model.
        /// </summary>
        public ListViewModel ListViewModel
        {
            get
            {
                return (ListViewModel)GetValue(ListViewModelProperty);
            }
            set
            {
                SetValue(ListViewModelProperty, value);
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes the instance of MainWindow.
        /// </summary>
        public MainWindow()
        {
            AddFontFamily();
            InitializeComponent();
            ListViewModel = new ListViewModel(this.richTextBoxAdv);
            this.Loaded += MainWindow_Loaded;
        }
        #endregion

        #region Static Dependency Properties
        // Using a DependencyProperty as the backing store for FontFamilySource.
        public static readonly DependencyProperty FontFamilySourceProperty = DependencyProperty.Register("FontFamilySource", typeof(List<string>), typeof(MainWindow), new PropertyMetadata(null));
        // Using a DependencyProperty as the backing store for FontSizeSource.
        public static readonly DependencyProperty FontSizeSourceProperty = DependencyProperty.Register("FontSizeSource", typeof(double[]), typeof(MainWindow), new PropertyMetadata(new double[] { 8, 9, 10, 11, 12, 14, 16, 18, 20, 24, 26, 28, 36, 48, 72 }));
        // Using a DependencyProperty as the backing store for ListViewModelSource.
        public static readonly DependencyProperty ListViewModelProperty = DependencyProperty.Register("ListViewModel", typeof(ListViewModel), typeof(MainWindow), new PropertyMetadata(null));
        #endregion

        #region Events
        /// <summary>
        /// Called when window loaded.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            richTextBoxAdv.Focus();
        }
        #endregion

        #region Implementation
        /// <summary>
        /// Adds the font family.
        /// </summary>
        private void AddFontFamily()
        {
            FontFamilySource = new List<string>();
            foreach (FontFamily fontfamily in Fonts.SystemFontFamilies)
            {
                FontFamilySource.Add(fontfamily.Source);
            }
        }
        /// <summary>
        /// Called on bulleted list split button clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An <see cref="T:System.Windows.RoutedEventArgs">RoutedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        void BulletedListSplitButton_Click(object sender, RoutedEventArgs e)
        {
            ListViewModel viewModel = (sender as FrameworkElement).DataContext as ListViewModel;
            if (string.IsNullOrEmpty(viewModel.ListName) || viewModel.ListName == "NoList" || viewModel.ListName == "Null" || !viewModel.ListName.StartsWith("_Bullet"))
                viewModel.ListName = "_Bullet_Dot";
            else
                viewModel.ListName = "NoList";
        }
        /// <summary>
        /// Called on numbered list split button clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An <see cref="T:System.Windows.RoutedEventArgs">RoutedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        void NumberedListSplitButton_Click(object sender, RoutedEventArgs e)
        {
            ListViewModel viewModel = (sender as FrameworkElement).DataContext as ListViewModel;
            if (string.IsNullOrEmpty(viewModel.ListName) || viewModel.ListName == "NoList" || viewModel.ListName == "Null" || !viewModel.ListName.StartsWith("_Numbering"))
                viewModel.ListName = "_Numbering_Number_Dot";
            else
                viewModel.ListName = "NoList";
        }
        /// <summary>
        /// Occurs when the DropDownButton clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An <see cref="T:System.Windows.RoutedEventArgs">RoutedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        private void DropDownItem_Click(object sender, RoutedEventArgs e)
        {
            object obj = (sender as FrameworkElement).Parent;
            while (!(obj is SplitButtonAdv))
            {
                obj = (obj as FrameworkElement).Parent;
            }
            if (obj is SplitButtonAdv)
                (obj as SplitButtonAdv).IsDropDownOpen = false;
        }
        #endregion
    }

    /// <summary>
    /// The list view model.
    /// </summary>
    /// <remarks></remarks>
    public class ListViewModel : INotifyPropertyChanged
    {
        #region Fields
        private string listName;
        private SfRichTextBoxAdv richTextBoxAdv = null;
        Dictionary<string, ListAdv> lists = null;
        private bool isRetrieving = false;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the ListName.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public string ListName
        {
            get
            {
                return listName;
            }
            set
            {
                if (value == listName)
                    return;
                listName = value;
                NotifyPropertyChanged("ListName");
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="T:DocumentEditor_WPF.ListViewModel">ListViewModel</see> class. 
        /// </summary>
        /// <param name="richTextBoxAdv"></param>
        /// <remarks></remarks>
        public ListViewModel(SfRichTextBoxAdv richTextBoxAdv)
        {
            if (richTextBoxAdv != null)
            {
                this.richTextBoxAdv = richTextBoxAdv;
                lists = new Dictionary<string, ListAdv>();
                List<string> listNames = new List<string>()
                {
                    "_Bullet_Dot", "_Bullet_Square", "_Bullet_Circle", "_Bullet_Flower", "_Bullet_Tick", "_Bullet_Arrow" ,
                    "_Numbering_LowLetter_Brace","_Numbering_LowLetter_Dot","_Numbering_Number_Brace","_Numbering_Number_Dot","_Numbering_UpLetter","_Numbering_UpRoman","_Numbering_LowRoman",
                };
                foreach (string listName in listNames)
                {
                    lists.Add(listName, CreateList(listName));
                }
                richTextBoxAdv.SelectionChanged += richTextBoxAdv_SelectionChanged;
            }
        }
        #endregion

        #region Events
        /// <summary>
        /// Occurs when Property Changed. 
        /// </summary>
        /// <remarks></remarks>
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// Occurs when the richTextBoxAdv selection changes
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args">An <see cref="T:Syncfusion.Windows.Controls.RichTextBoxAdv.SelectionChangedEventArgs">SelectionChangedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        void richTextBoxAdv_SelectionChanged(object obj, Syncfusion.Windows.Controls.RichTextBoxAdv.SelectionChangedEventArgs args)
        {
            isRetrieving = true;
            ListAdv list = richTextBoxAdv.Selection.ParagraphFormat.GetList();
            if (list == null)
            {
                if (richTextBoxAdv.Selection.ParagraphFormat.ListLevelNumber == 0)
                    ListName = "NoList";
                else
                    ListName = "Null";
            }
            else
            {
                list = richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.List;
                bool hasList = false;
                foreach (string name in lists.Keys)
                {
                    if (lists[name] == list)
                    {
                        ListName = name;
                        hasList = true;
                        break;
                    }
                }
                if (!hasList)
                    ListName = "List";
            }
            isRetrieving = false;
        }
        #endregion

        #region Implementation
        /// <summary>
        /// Notifies when the property changes
        /// </summary>
        /// <param name="propertyName"></param>
        /// <remarks></remarks>
        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            if (!isRetrieving && richTextBoxAdv != null)
            {
                if (listName == "NoList")
                    richTextBoxAdv.Selection.ParagraphFormat.SetList(null);
                if (lists.ContainsKey(ListName))
                    richTextBoxAdv.Selection.ParagraphFormat.SetList(lists[ListName]);
            }
        }
        /// <summary>
        /// Creates the list
        /// </summary>
        /// <param name="listName"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        internal ListAdv CreateList(string listName)
        {
            AbstractListAdv abstractListAdv = new AbstractListAdv(null);
            ListAdv list = new ListAdv(null);
            list.AbstractList = abstractListAdv;
            if (listName.StartsWith("_List"))
            {
                AddListLevels(abstractListAdv, listName);
            }
            else
            {
                ListLevelAdv listLevel = new ListLevelAdv(abstractListAdv);
                abstractListAdv.Levels.Add(listLevel);
                if (listName.StartsWith("_Bullet"))
                {
                    listLevel.ListLevelPattern = ListLevelPattern.Bullet;
                    switch (listName.Substring(8))
                    {
                        case "Dot":
                            listLevel.NumberFormat = "\uf0b7";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Symbol");
                            break;
                        case "Square":
                            listLevel.NumberFormat = "\uf0a7";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                        case "Circle":
                            listLevel.NumberFormat = "\uf06f" + "\u0020";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Symbol");
                            break;
                        case "Flower":
                            listLevel.NumberFormat = "\uf076";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                        case "Arrow":
                            listLevel.NumberFormat = "\uf0d8";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                        case "Tick":
                            listLevel.NumberFormat = "\uf0fc";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                    }
                }
                else
                {
                    if (listName.Contains("UpRoman"))
                        listLevel.ListLevelPattern = ListLevelPattern.UpRoman;
                    else if (listName.Contains("LowRoman"))
                        listLevel.ListLevelPattern = ListLevelPattern.LowRoman;
                    else if (listName.Contains("UpLetter"))
                        listLevel.ListLevelPattern = ListLevelPattern.UpLetter;
                    else if (listName.Contains("LowLetter"))
                        listLevel.ListLevelPattern = ListLevelPattern.LowLetter;
                    else if (listName.Contains("Number"))
                        listLevel.ListLevelPattern = ListLevelPattern.Number;
                    if (listName.EndsWith("Brace"))
                        listLevel.NumberFormat = "%1)";
                    else
                        listLevel.NumberFormat = "%1.";
                    listLevel.StartAt = 1;
                }
                listLevel.CharacterFormat.FontColor = Colors.Black;
                listLevel.CharacterFormat.FontSize = 14;
                listLevel.ParagraphFormat.LeftIndent = 48;
                listLevel.ParagraphFormat.FirstLineIndent = -24;
            }
            return list;
        }
        /// <summary>
        /// Adds the list levels
        /// </summary>
        /// <param name="abstractListAdv"></param>
        /// <param name="listName"></param>
        /// <remarks></remarks>
        internal void AddListLevels(AbstractListAdv abstractListAdv, string listName)
        {
            List<string> bulletCharacters = new List<string>() { "\uf076", "\uf0d8", "\uf0a7", "\uf0b7", "\uf0a8" };
            for (int i = abstractListAdv.Levels.Count; i < 9; i++)
            {
                ListLevelAdv listLevel = new ListLevelAdv(abstractListAdv);
                if (listName.Contains("Bullet"))
                {
                    listLevel.ListLevelPattern = ListLevelPattern.Bullet;
                    listLevel.NumberFormat = bulletCharacters[i < 5 ? i % 5 : i % 5 + 1];
                    listLevel.CharacterFormat.FontFamily = i < 3 || i == 5 ? new FontFamily("Wingdings") : new FontFamily("Symbol");
                }
                else
                {
                    if (listName.Contains("Multilevel"))
                    {
                        for (int j = 0; j < i + 1; j++)
                            listLevel.NumberFormat += "%" + (j + 1).ToString() + ".";
                        listLevel.ListLevelPattern = ListLevelPattern.Number;
                    }
                    else
                    {
                        listLevel.NumberFormat = "%" + (i + 1).ToString() + ")";
                        listLevel.ListLevelPattern = i % 3 == 0 ? ListLevelPattern.Number
                       : i % 3 == 1 ? ListLevelPattern.LowLetter : ListLevelPattern.LowRoman;
                    }
                    listLevel.StartAt = 1;
                    listLevel.RestartLevel = i;
                }
                listLevel.CharacterFormat.FontColor = Colors.Black;
                listLevel.CharacterFormat.FontSize = 14;
                listLevel.ParagraphFormat.LeftIndent = 48 * i;
                listLevel.ParagraphFormat.FirstLineIndent = -24;
                abstractListAdv.Levels.Add(listLevel);
            }
        }
        /// <summary>
        /// Releases all resources used by the.
        /// <see cref="T:DocumentEditor_WPF.ListViewModel">ListViewModel</see>.
        /// </summary>
        /// <remarks></remarks>
        internal void Dispose()
        {
            listName = null;
            if (lists != null)
            {
                lists.Clear();
                lists = null;
            }
            if (richTextBoxAdv != null)
            {
                richTextBoxAdv.SelectionChanged -= richTextBoxAdv_SelectionChanged;
                richTextBoxAdv = null;
            }
        }
        #endregion
    }

    /// <summary>
    /// Specifies the List value to Toggle converter.
    /// </summary>
    /// <remarks></remarks>
    internal class ListToggleConverter : IValueConverter
    {
        /// <summary>
        /// Converts the List value to Toggle.
        /// </summary>
        /// <param name="value">The value produced by the binding source.</param>
        /// <param name="targetType">The type of the binding target property.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value != null && parameter != null && value.ToString() == parameter.ToString())
                return true;
            return false;
        }
        /// <summary>
        /// Converts the Toggle value to List.
        /// </summary>
        /// <param name="value">The value that is produced by the binding target.</param>
        /// <param name="targetType">The type to convert to.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (parameter == null)
                return null;
            return parameter.ToString();
        }
    }
}
